import random
import os.path
import hashlib
from uuid import uuid4

from django.conf import settings
from django.core.files.storage import FileSystemStorage


def get_random_filename(instance, filename):
    """
    Generates random filename for uploading file using uuid4 hashes
    You need to define UPLOADS_ROOT in your django settings
    something like this
    UPLOADS_ROOT = rel(MEDIA_ROOT, 'uploads')
     """
    folder = settings.UPLOADS_ROOT
    ext = filename.split('.')[-1]
    filename = '{}.{}'.format(str(uuid4()), ext)
    return os.path.join(folder, filename)


def image_path(instance, filename):
    """Generates likely unique image path using md5 hashes"""
    filename, ext = os.path.splitext(filename.lower())
    instance_id_hash = hashlib.md5(str(instance.id)).hexdigest()
    filename_hash = ''.join(random.sample(hashlib.md5(filename.encode('utf-8')).hexdigest(), 8))
    return '{}/{}{}'.format(instance_id_hash, filename_hash, ext)


storage_factory = lambda root, url: FileSystemStorage(location=root, base_url=url)



import magic

from django import forms
from django.utils.translation import ugettext_lazy as _


class TypedFileField(forms.FileField):

    """
    Field for validation type of uploaded file
    Field take either "allowed_mimes" or "allowed_exts" lists
    with mimes or exts respectively

    Max size taking MB(megabytes)

    Example:
    file = TypedFileField(allowed_mimes=['application/pdf',
                                         'image/png', 'image/jpeg',
                                         'application/msword',
                                         'application/zip',
                                         'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
                          allowed_exts=['doc',
                                        'docx',
                                        'jpeg',
                                        'jpg',
                                        'png',
                                        'pdf'],
                          max_size=10)

    Or you can define FILE_UPLOAD_MAX_MEMORY_SIZE_CUSTOM and using is like,
    max_size=settings.FILE_UPLOAD_MAX_MEMORY_SIZE_CUSTOM
    """

    def __init__(self, *args, **kwargs):
        self.allowed_mimes = kwargs.pop('allowed_mimes', '')
        self.allowed_exts = kwargs.pop('allowed_exts', '')
        self.max_size = kwargs.pop('max_size', None)
        super(TypedFileField, self).__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        value = super(TypedFileField, self).clean(*args, **kwargs)
        if value and (self.allowed_mimes or self.allowed_exts):
            mime = magic.from_buffer(value.read(1024), mime=True)
            ext = value.name.split('.')[-1].lower()
            if (mime not in self.allowed_mimes) or (ext not in self.allowed_exts):
                raise forms.ValidationError(_('This file extension does not support'))
        if value and self.max_size:
            if value.size/1024.0**2 > self.max_size:
                raise forms.ValidationError(_('This file is too big, max size {} MB'.format(self.max_size)))
        return value



from functools import wraps

from django.core.cache import cache
from django.http import HttpResponseBadRequest
from django.core.exceptions import ObjectDoesNotExist


def get_cache_key(prefix, *args, **kwargs):
    """
    Calculates cache key based on `args` and `kwargs`.
    `args` and `kwargs` must be instances of hashable types.
    """
    hash_args_kwargs = hash(tuple(kwargs.iteritems()) + args)
    return '{}_{}'.format(prefix, hash_args_kwargs)


def cache_method(func=None, prefix=''):
    """
    Cache result of function execution into the `self` object (mostly useful in models).
    Calculate cache key based on `args` and `kwargs` of the function (except `self`).
    """
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            cache_key_prefix = prefix or '_cache_{}'.format(func.__name__)
            cache_key = get_cache_key(cache_key_prefix, *args, **kwargs)
            if not hasattr(self, cache_key):
                setattr(self, cache_key, func(self))
            return getattr(self, cache_key)
        return wrapper
    if func is None:
        return decorator
    else:
        return decorator(func)


def cache_func(prefix, method=False):
    """
    Cache result of function execution into the django cache backend.
    Calculate cache key based on `prefix`, `args` and `kwargs` of the function.
    For using like object method set `method=True`.
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            cache_args = args
            if method:
                cache_args = args[1:]
            cache_key = get_cache_key(prefix, *cache_args, **kwargs)
            cached_value = cache.get(cache_key)
            if cached_value is None:
                cached_value = func(*args, **kwargs)
                cache.set(cache_key, cached_value)
            return cached_value
        return wrapper
    return decorator


def get_or_default(func=None, default=None):
    """
    Wrapper around Django's ORM `get` functionality.
    Wrap anything that raises ObjectDoesNotExist exception
    and provide the default value if necessary.
    `default` by default is None. `default` can be any callable,
    if it is callable it will be called when ObjectDoesNotExist
    exception will be raised.
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except ObjectDoesNotExist:
                if callable(default):
                    return default()
                else:
                    return default
        return wrapper
    if func is None:
        return decorator
    else:
        return decorator(func)


def ajax_required(func):
    @wraps(func)
    def wrapper(request, *args, **kwargs):
        if request.method == 'POST' and request.is_ajax():
            return func(request, *args, **kwargs)
        return HttpResponseBadRequest()
    return wrapper



import json

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.decorators import login_required
from django.core.serializers.json import DjangoJSONEncoder

from .decorators import ajax_required


class LoginRequiredMixin(object):

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(LoginRequiredMixin, self).dispatch(*args, **kwargs)


class AjaxRequiredMixin(object):

    @method_decorator(ajax_required)
    def dispatch(self, *args, **kwargs):
        return super(AjaxMixin, self).dispatch(*args, **kwargs)


class AjaxMixin(AjaxRequiredMixin):
    """
    A mixin that raise 400 if request was not POST or was made not using ajax.
    Also render response as json.
    """
    response_class = HttpResponse

    def render_to_response(self, context=None, **response_kwargs):
        """
        Return a JSON response, transforming 'context' to make the payload.
        """
        response_kwargs['content_type'] = 'application/json'
        return self.response_class(self.convert_context_to_json(context or {}), **response_kwargs)

    def convert_context_to_json(self, context):
        """
        Convert the context dictionary into a JSON object
        """
        return json.dumps(context, cls=DjangoJSONEncoder)


class ActiveTabMixin(object):
    """
    Mixin to set active tab menu
    """
    active_tab = None

    def get_active_tab(self):
        if self.active_tab is None:
            raise ImproperlyConfigured(
                "ActiveTabMixin requires either a definition of "
                "'active_tab' or an implementation of 'get_active_tab()'")
        return self.active_tab

    def get_context_data(self, **kwargs):
        context = super(ActiveTabMixin, self).get_context_data(**kwargs)
        context['active_tab'] = self.get_active_tab()
        return context


class ReadOnlyAdminMixin(object):
    """
    Mixin for ModelAdmin to make it read only.
    """
    actions = None

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False if obj is not None else True

    def has_delete_permission(self, request, obj=None):
        return False

    def get_readonly_fields(self, request, obj=None):
        if obj is not None:
            return tuple(f.name for f in obj._meta.fields)
        return super(ReadOnlyAdminMixin, self).get_readonly_fields(request, obj)

    def __init__(self, *args, **kwargs):
        super(ReadOnlyAdminMixin, self).__init__(*args, **kwargs)
        self.list_display_links = (None, )



import os
import setuptools


SRC_ROOT = os.path.abspath(os.path.dirname(__file__))


with open(os.path.join(SRC_ROOT, 'requirements.txt')) as f:
    required = f.read().splitlines()


setuptools.setup(
    name="django-skd-tools",
    version="0.0.9",
    author="Steelkiwi",
    author_email="vilisov@steelkiwi.com",  # temp
    url="https://github.com/steelkiwi/django-skd-tools",
    license="MIT",
    description="Steelkiwi Django Tools",
    keywords="django tools helpers",
    packages=setuptools.find_packages(),
    install_requires=required,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    # tests_require=[],
    # test_suite='tests.runtests.runtests',
)
